import os
import json
import web3

# HERE LOCATION
HERE = os.path.dirname(os.path.abspath(__file__))


class Bifrost:

    def __init__(self, factory, signer_account, provider):
        self.w3 = web3.Web3(provider)
        abiFactory = json.load(
            open(os.path.join(HERE, 'abi/Bifrost.json'), 'r')
        )
        self.factory = self.w3.eth.contract(
            address=factory,
            abi=abiFactory
        )
        self.abiProxy = json.load(
            open(os.path.join(HERE, 'abi/BifrostProxy.json'), 'r')
        )
        self.signer = signer_account
    
    def getAddress(self, salt, signer_account=None):
        _signer = self.signer if signer_account is None else signer_account
        address = self.factory.functions.getDeploymentAddress(salt, _signer.address).call()
        return self.w3.toChecksumAddress(address)

    def isDeployed(self, salt, signer_account=None):
        address = self.getAddress(salt, signer_account)
        proxy = self.w3.eth.contract(
            address=address,
            abi=self.abiProxy
        )
        try:
            exist = proxy.functions.exist().call()
        except web3.exceptions.BadFunctionCallOutput:
            exist = False
        return exist

    def makeCall(self, salt, destination, value, data=None, signer_account=None):
        _signer = self.signer if signer_account is None else signer_account
        _data = b'' if data is None else data
        # Generate TX
        tx = self.factory.functions.makeCall(salt, destination, value, _data).buildTransaction({
            'nonce': self.w3.eth.getTransactionCount(_signer.address),
            'from': _signer.address
        })
        # Sign TX
        signed_tx = self.signer.signTransaction(tx)
        # Send TX
        return self.w3.eth.sendRawTransaction(signed_tx.rawTransaction)



import setuptools
import pathlib

HERE = pathlib.Path(__file__).parent

README = (HERE / 'README.md').read_text()

setuptools.setup(
    name='bifrost_py',
    version='0.0.5',
    author="Ignacio Althabe",
    author_email="nacho.althabe@gmail.com ",
    description="Wallets based on SALT instead of Nonce.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://gitlab.riaquest.com/nacho/bifrost",
    packages=['bifrost_py'],
    package_data = {
        'bifrost_py': ['**/*']
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)



from web3 import Web3
import json
from bifrost import Bifrost

SIGNER_PK = "b961c45b00dbc70e11cacc8e9ae15cd889ac98df2e38ec2fcafc37ddbe31721e"
FACTORY_ADDRESS = "0xb442b6b2F19cd70799B3987DfA069a8b7c9EA6B3"
RPC_URL = "https://rinkeby.infura.io/v3/"
BIFROST_FACTORY = "0xb442b6b2F19cd70799B3987DfA069a8b7c9EA6B3"
ERC20_ADDRESS = "0x46324Fde2E0347aEa72BA2A085b30E68556ac294"

try:
    from local_settings import *
except ModuleNotFoundError:
    pass

w3Provider = Web3.HTTPProvider(RPC_URL)
w3 = Web3(w3Provider)
signer_account = w3.eth.account.privateKeyToAccount(SIGNER_PK)

bifrost = Bifrost(BIFROST_FACTORY, signer_account, w3Provider)

address_salt = 3
address = bifrost.getAddress(address_salt)
address_is_deployed = bifrost.isDeployed(address_salt)
print('''
    Signer: %s
    Salt: %s
    Address: %s
    IsDeployed: %s 
''' % (signer_account.address, address_salt, address, address_is_deployed))

amount = int(.1 * 10 ** 18)

# Transfer ETH
# address shoul have more than .1 (amount) ETH
tx = bifrost.makeCall(address_salt, signer_account.address, amount)
print('ETH transfer: ', tx.hex())
w3.eth.waitForTransactionReceipt(tx)

# Transfer ERC20
# address shoul have more than .1 (amount) ERC20
ERC20_abi = json.load(open('./abi/IERC20.json', 'r'))
ERC20 = w3.eth.contract(

    abi=ERC20_abi
)
call = ERC20.encodeABI('transfer', args=(signer_account.address, amount))
tx = bifrost.makeCall(address_salt, ERC20_ADDRESS, 0, call)
print('ERC20 transfer: ', tx.hex())


